@@ -6,16 +6,12 @@ Apache2::Controller::Auth::OpenID - OpenID for Apache2::Controller
 
 =head1 VERSION
 
-Version 1.000.001 - FIRST RELEASE
+Version 1.000.010 - THIS MODULE DISABLED, DOES NOT WORK.
 
 =cut
 
 use version;
-our $VERSION = version->new('1.000.001');
-
-=head1 UNIMPLEMENTED
-
-UNIMPLEMENTED!  Peril!  Pitfalls!  Pernicious knids!
+our $VERSION = version->new('1.000.010');
 
 =head1 SYNOPSIS
 
@@ -64,9 +60,14 @@ UNIMPLEMENTED!  Peril!  Pitfalls!  Pernicious knids!
      # your own constant salt of arbitrary length
      A2C_Auth_OpenID_Consumer_Secret    abcdefg1234567
 
+     # if you do not want to preserve GET/POST params 
+     # across redirects to the OpenID server, use this flag:
+     # A2C_Auth_OpenID_NoPreserveParams
+
+     # if you do not overload get_uname() (see below), then
      # PerlHeaderParserHandlers must be invoked in order
-     # to set up the session and dbi handle before checking auth
-     # In this example,
+     # to set up the dbi handle before checking auth
+     # with the default method.  In this example,
      # MyApp::DBI::Connector is an Apache2::Controller::DBI::Connector
      # and MyApp::Session is an Apache2::Controller::Session::Cookie...
      # see those modules for more info.
@@ -83,7 +84,7 @@ Implements an authentication mechanism for L<Apache2::Controller>
 that uses OpenID.  
 
 This is NOT an AuthenPerlHandler.  This is an implementation
-of a simple cookie-based session mechanism that shows the browser
+of a simple cookie-based mechanism that shows the browser
 a login page, where your controller should present and process an
 HTML form for logging in.  
 
@@ -105,7 +106,7 @@ and C<< {a2c}{openid}{errcode} >>.
 =head2 REDIRECTION OR REDISPATCH?
 
 Whether redirecting or redispatching, stuff has to be saved
-in the session, so C<< $r->notes->{a2c_session_force_save >>
+in the session, so C<< $r->notes->{a2c}{session_force_save} >>
 will be set.
 
 =head3 INTERNAL LOGIN, LOGOUT AND REGISTER PAGES
@@ -116,7 +117,7 @@ If the uris for these pages are relative, not absolute, i.e.
 they are handled by the same controller that we're going to
 anyway, then it
 tries setting the uri and re-dispatching by grabbing the dispatch
-class name out of C<< $r->notes->{a2c_dispatch_class} >> and 
+class name out of C<< $r->pnotes->{a2c}{dispatch_class} >> and 
 instantiating a new dispatch handler object.
 
 (Dispatch can't keep the handler
@@ -154,22 +155,31 @@ and should preserve this for the return url.  It uses
 C<< $r->construct_url() >> as the trusted root.  
 
 When the user passes the authentication process (C<< $csr->verified_identity >>),
-it sets C<< $r->notes->{a2c_openid_logged_in} >> for this request
+it sets C<< $r->pnotes->{a2c}{openid_logged_in} >> for this request
 to let your handler know if you want to display a message like 
 "You have successfully logged in" or something.
 
-=head3 POST VARS
+=head3 GET AND POST VARS
 
-not yet...
+The whole point of OpenID is that the login mechanism is
+invisible - as long as the user can claim to own the url,
+and the auth server returns a positive response, then the
+user's session should continue.
 
-=for comment
+So, this preserves GET parmas and the POST body through the 
+login process and the redirect sequence to the OpenID server,
+including when the local session times out.  If they come back
+after a while and click a submit button, but either their
+local session has timed out or their OpenID server session
+has timed out through whatever mechanism that uses, then after
+they log into OpenID and are redirected back to the protected
+area, the GET params and POST body are restored, and it will
+do what the user expected when they clicked the submit button.
 
-If redirected, it also stashes the POST body into
-the session as C<< {a2c}{openid}{redirect_post} >>
-but if you use this, you probably want to put some kind of form id
-and a unique value for every post so you're sure you're
-doing the right thing.  I don't know if this will work,
-but it might be useful.
+This behavior
+is a feature, so it is enabled by default, but it may not
+be expected, so you can turn it off by using the directive
+flag C<< A2C_Auth_OpenID_NoPreserveParams >>.
 
 =head1 DIRECTIVES
 
@@ -187,27 +197,12 @@ I have heard there are trickier things one can do to ensure the
 security of a session based cookie.  This module just implements
 a simple association of a user with a session key by storing a flag
 and a last-accessed time value in the session hash, nothing fancier.
+If you have recommendations, please let me know.
 
 This calls C<< $r->connection->get_remote_host >> and saves it
 in the C<< {a2c}{openid} >> section of the session hash.  So if
 you don't want it to do DNS lookups, set directive C<< HostNameLookups off >>.
 
-=head1 METHODS
-
-"These aren't the methods you're looking for."
-
-"These aren't the methods we're looking for."
-
-"He can go about his business."
-
-"You can go about your business."
-
-"Move along."
-
-"Move along.  Move along."
-
-You don't access these methods.  This is internal documentation.
-
 =cut
 
 use strict;
@@ -231,9 +226,83 @@ use Apache2::Const -compile => qw( OK SERVER_ERROR REDIRECT );
 
 use Apache2::Controller::X;
 
+=head2 new
+
+Overloaded constructor will always throw an L<Apache2::Controller::X>
+because this module does not work.
+
+=cut
+
+sub new {
+    a2cx __PACKAGE__." is disabled, does not work, do not use.";
+}
+
 # hopefully we get the same default consumer secret as in top level
 use Apache2::Controller::Const qw( $DEFAULT_CONSUMER_SECRET );
 
+=head1 OVERLOADABLE METHODS
+
+The only method which should be overloaded in your subclass
+is C<< get_uname( $openid_url ) >> which returns the username
+string that corresponds to the openid url supplied by the cookie.
+When overloading, you get the RequestRec in C<< $self->{r} >>.
+
+=head2 get_uname
+
+ my $uname = $self->get_uname($openid_url);
+
+Takes a string which is the supplied openid_url.
+You can overload C<< get_uname >> to supply it
+by some other means, such as by LDAP.
+
+=cut
+
+sub get_uname {
+    my ($self, $openid_url) = @_;
+
+    a2cx "get_uname() requires an openid_url string param"
+        if !$openid_url || ref $openid_url;
+
+    my $conf = $self->{conf};
+
+    my $pnotes = $self->pnotes;
+    DEBUG sub { "pnotes: ".Dump($pnotes) };
+
+    my $dbh = $self->pnotes->{a2c}{ $conf->{dbi_name} }
+        || a2cx "Database handle '$conf->{dbi_name}' is not connected in pnotes"
+            ." for default handler ".__PACKAGE__;
+
+    my $uname;
+    eval {
+        ($uname) = $dbh->selectrow_array(
+            qq| SELECT  $conf->{user_field} 
+                FROM    $conf->{table} 
+                WHERE   $conf->{url_field} = ? 
+            |, undef, $openid_url
+        );
+    };
+    a2cx "Error in default get_uname() from dbh: $EVAL_ERROR"
+        if $EVAL_ERROR;
+
+    return $uname;
+}
+
+=head1 INTERNAL METHODS
+
+"These aren't the methods you're looking for."
+
+"These aren't the methods we're looking for."
+
+"He can go about his business."
+
+"You can go about your business."
+
+"Move along."
+
+"Move along.  Move along."
+
+You don't access these methods.  This is internal documentation.
+
 =head2 default_directives 
 
 Calculate and return the hash of directive defaults.
@@ -258,6 +327,7 @@ sub default_directives {
             timeout         => 20,
         },
         consumer_secret => $DEFAULT_CONSUMER_SECRET,
+        nopreserveparams => 0,
     );
 }
 
@@ -291,7 +361,7 @@ If uri = logout uri, delete session hash login flags and return OK.
 
 sub _save_errs_in_sess {
     my ($self, $openid_csr) = @_;
-    my $sess = $self->pnotes->{session};
+    my $sess = $self->pnotes->{a2c}{session};
     my $errcode = $openid_csr->errcode 
         || $openid_csr->{last_errcode} || '[ no error code ]';
     my $errtext = $openid_csr->err 
@@ -308,8 +378,11 @@ sub process {
     my ($self) = @_;
     my $uri = $self->uri();
 
+  # my $pnotes = $self->pnotes;
+  # DEBUG sub { "Before checking session, pnotes is:\n".Dump($pnotes) };
+
     # make sure a session object is set up already
-    my $sess = $self->pnotes('session')
+    my $sess = $self->pnotes->{a2c}{session}
         || a2cx "No session object configured for handler";
 
     DEBUG sub { "Entering, processing uri '$uri'.\nsession is:\n".Dump($sess) };
@@ -322,6 +395,7 @@ sub process {
             Login       Logout      Register    Timeout
             Table       User_Field  URL_Field   DBI_Name    
             Trust_Root  LWP_Class   Allow_Login Consumer_Secret
+            NoPreserveParams
         ) ),
     );
 
@@ -331,15 +405,14 @@ sub process {
     # slap in anything specified in the sub-hash of LWP class options
     my $lwp_opts_directive = $directives->{A2C_Auth_OpenID_LWP_Opts} || { };
     my @lwp_opt_keys = keys %{$lwp_opts_directive};
-    DEBUG sub { "Trying to slice in opts: ".Dump($lwp_opts_directive) };
-    if (scalar @lwp_opt_keys) {
-        @{$conf{lwp_opts}}{@lwp_opt_keys} = @{$lwp_opts_directive}{@lwp_opt_keys};
-    }
+    DEBUG sub { "Trying to slice in lwp_opts: ".Dump($lwp_opts_directive) };
+    @{$conf{lwp_opts}}{@lwp_opt_keys} = @{$lwp_opts_directive}{@lwp_opt_keys}
+        if scalar @lwp_opt_keys;
 
-    $self->{a2c_openid_conf} = \%conf;
+    $self->{conf} = \%conf;
 
-    DEBUG sub { "a2c_openid_conf:\n".Dump(\%conf) };
-    DEBUG sub { "session:\n".Dump($self->pnotes->{session}) };
+    DEBUG sub { "conf:\n".Dump(\%conf) };
+    DEBUG sub { "session:\n".Dump($self->pnotes->{a2c}{session}) };
 
     # if we're on the register page, allow it through
     return Apache2::Const::OK if $uri eq $self->qualify_uri($conf{register});
@@ -367,10 +440,9 @@ sub process {
             ."$EVAL_ERROR" if $EVAL_ERROR;
 
         # stash string in package to avoid closure circle on this req's %conf.
-        # we provide some junk if they didn't use the directive to specify
-        # or generate some.
-        $consumer_secret_string = $conf{consumer_secret} 
-            || q|-qf_AD4#~a{~3)84cCvd+$6R89+,[l|;   
+        # we provide some hardcoded junk if they didn't use the directive 
+        # to specify or generate some.
+        $consumer_secret_string = $conf{consumer_secret};
         DEBUG "Setting up CSR with secret string '$consumer_secret_string'";
 
         $openid_csr = Net::OpenID::Consumer->new(
@@ -432,19 +504,15 @@ sub process {
 
     DEBUG "looking for uname from openid table using openid_url '$openid_url'";
 
-    my $dbh = $self->pnotes->{ $conf{dbi_name} }
-        || a2cx "Database handle '$conf{dbi_name}' is not connected in pnotes.";
-
-    my ($uname) = $dbh->selectrow_array(
-        q{ SELECT uname FROM openid WHERE openid_url = ? },
-        undef, $openid_url
-    );
+    my $uname = $self->get_uname($openid_url);
 
     if (!$uname) {
         DEBUG("no uname! ... ".(defined $uname ? "'$uname'" : '[undef]'));
         return $self->redirect_to($conf{register});
     }
 
+    $self->{uname} = $uname;
+
     DEBUG "Trying authentication for known user: $uname, $openid_url";
     # okay, handle the authentication
 
@@ -482,20 +550,31 @@ sub process {
 
         my $connection = $self->connection;
 
+        my $openid_sess = $sess->{a2c}{openid} ||= { };
+
         # update the session
-        $sess->{a2c}{openid}{logged_in} = 1;
-        $sess->{a2c}{openid}{last_accessed_time} = time;
-        $sess->{a2c}{openid}{remote_host} = $connection->get_remote_host();
-        $sess->{a2c}{openid}{remote_ip}   = $connection->remote_ip();
-        $sess->{a2c}{openid}{openid_url}  = $openid_url;
+        $openid_sess->{logged_in} = 1;
+        $openid_sess->{last_accessed_time} = time;
+        $openid_sess->{remote_host} = $connection->get_remote_host();
+        $openid_sess->{remote_ip}   = $connection->remote_ip();
+        $openid_sess->{openid_url}  = $openid_url;
+
+        # restore the saved query params and post body
+        $self->args($openid_sess->{previous}{get_args})
+            if $openid_sess->{previous}{get_args};
+
+        if (my $post_params = $openid_sess->{previous}{post_params}) {
+        }
 
         # if everything works and we're returning the user okay,
         # make sure to delete the previous url from session hash
-        delete $sess->{a2c}{openid}{previous_uri};
+        delete $openid_sess->{previous};
 
         # set a flag for this request so controller can print
         # a "login successful" message
-        $self->notes->{a2c_openid_logged_in} = 1;
+        $self->pnotes->{a2c}{openid_logged_in} = 1;
+
+        $self->user($uname);
 
         return Apache2::Const::OK;
     }
@@ -504,18 +583,23 @@ sub process {
         # figure out what uri they're returning to.
         # if we can't figure one out from session,
         # they go back to the login page.
-        my $return_uri = $sess->{a2c}{openid}{previous_uri};
+        my $return_uri = $sess->{a2c}{openid}{previous}{uri};
         DEBUG "previous_uri from sess is ".($return_uri || '[none]');
 
         my %qual_uris = map {($_ => $self->qualify_uri($conf{$_}))} 
             qw( login logout register );
 
-        $return_uri 
+        # depending on what the return uri was supposed to be, set
+        # it, or maybe the login uri, or maybe the current uri
+        my $real_return_uri 
           = $return_uri                     ? $return_uri
           : $uri eq $qual_uris{logout}      ? $qual_uris{login} # wrong
           : $uri;
 
-        my $return_to = $self->construct_url($return_uri);
+        # make sure we save the uri in any case?
+        $sess->{a2c}{openid}{previous}{uri} = $real_return_uri;
+
+        my $return_to = $self->construct_url($real_return_uri);
         DEBUG "calling claimed_identity->check_url with return_to '$return_to'";
         DEBUG sub { "openid part of session is ".Dump($sess->{a2c}{openid}) };
 
@@ -543,6 +627,31 @@ sub process {
     }
 }
 
+=head2 DESTROY
+
+To save memory and be clean, when the object is destroyed,
+the package-space var C<< %params_hash >> is cleared.
+
+C<< process() >> has to populate a package space variable 
+C<< %params_hash >>
+with the params contents, so we won't create a closure
+that includes the request object when we construct the
+params subroutine for the cached openid CSR object.  Otherwise,
+the CSR keeps a reference to our handler object around,
+which contains a reference to the request object, and
+then neither the request nor the handler object are 
+cleaned up after this handler exits.  (Apparently
+Apache doesn't execute DESTROY until the next time it
+has to run this handler.  Doesn't quite make sense,
+but that's the way it behaved.)
+
+=cut
+
+sub DESTROY {
+    my ($self) = @_;
+    %params_hash = ();
+}
+
 =head2 qualify_uri
 
 If the uri is relative, qualifies it by prepending current location.
@@ -571,7 +680,7 @@ sub redirect_to {
     my ($self, $where_uri) = @_;
     a2cx "Undefined redirect" if !defined $where_uri;
 
-    my $conf = $self->{a2c_openid_conf};
+    my $conf = $self->{conf};
     my $current_uri = $self->uri();
 
     return exists $conf->{is_internal}{$where_uri}
@@ -597,7 +706,7 @@ sub location_redirect {
     $self->err_headers_out->add( Location => $uri );
 
     # set the flag to force the session to be saved
-    $self->notes->{a2c_session_force_save} = 1;
+    $self->pnotes->{a2c}{session_force_save} = 1;
 
     return Apache2::Const::REDIRECT;
 }
@@ -612,7 +721,11 @@ set location and return redirect.
 
 If where == register or login, and the current uri is not
 register or login, stash the current uri in
-C<< session->{a2c}{openid}{previous_uri} >>.
+C<< session->{a2c}{openid}{previous}{uri} >>, and if
+C<< A2C_Auth_OpenID_NoPreserveParams >> is NOT set,
+then it stashes the get args and post body in C<< ...{previous}{get} >>
+and C<< ...{previous}{post} >> for reattaching to the request
+after successful authentication on the return from the auth server.
 
 =cut
 
@@ -626,13 +739,13 @@ sub redispatch {
         if $where_uri =~ m{ \A / }mxs 
         || $where_uri =~ m{ \A \w+ :// }mxs;
 
-    my $conf = $self->{a2c_openid_conf};
+    my $conf = $self->{conf};
     my $current_uri = $self->uri();
     my $current_loc = $self->location();
     (my $current_relative_uri = $current_uri) =~ s{ \A \Q$current_loc\E / }{}mxs;
     DEBUG "current_relative_uri '$current_relative_uri'";
 
-    # save the current uri in session 
+    # save the current uri, get vars and post body in session 
     my $register_uri = $conf->{register};
     my $login_uri    = $conf->{login};
     if  (   ($where_uri eq $register_uri || $where_uri eq $login_uri)
@@ -640,7 +753,8 @@ sub redispatch {
         &&  $current_relative_uri ne $login_uri
         ) {
         DEBUG "setting session previous_uri to '$current_uri'";
-        $self->pnotes->{session}{a2c}{openid}{previous_uri} = $current_uri;
+        $self->pnotes->{a2c}{session}{a2c}{openid}{previous}{uri}  = $current_uri;
+        $self->preserve_params unless $conf->{nopreserveparams};
     }
 
     # now set the new URI and redispatch.
@@ -657,8 +771,8 @@ sub redispatch {
 
     $self->uri($where_full_uri);
 
-    my $dispatch_class = $self->notes->{a2c_dispatch_class}
-        || a2cx 'No dispatch class saved in $r->notes->{a2c_dispatch_class}';
+    my $dispatch_class = $self->pnotes->{a2c}{dispatch_class}
+        || a2cx 'No dispatch class saved in $r->pnotes->{a2c}{dispatch_class}';
     
   # # clear the previously set response handler
 
@@ -669,14 +783,14 @@ sub redispatch {
     # which seems like it should work, but in fact stalls the request
     # after the last cleanup handler completes, even though it seems
     # like everything completed successfully.
-    my $previously_set_controller = $self->notes->{controller}
-        || a2cx "no controller previously dispatched in notes->{controller}";
+    my $previously_set_controller = $self->pnotes->{a2c}{controller}
+        || a2cx "no controller previously dispatched in pnotes->{a2c}{controller}";
 
     eval {
         my $redispatch_handler = $dispatch_class->new($self->{r});
         $redispatch_handler->find_controller;
-        my $redispatch_controller = $self->notes->{controller}
-            || a2cx "Redispatch set no new controller in notes->{controller}";
+        my $redispatch_controller = $self->pnotes->{a2c}{controller}
+            || a2cx "Redispatch set no new controller in pnotes->{a2c}{controller}";
         a2cx "Redispatch controller '$redispatch_controller' is not previously "
             ."set controller '$previously_set_controller'"
             if $redispatch_controller ne $previously_set_controller;
@@ -694,6 +808,36 @@ sub redispatch {
     return Apache2::Const::OK;
 }
 
+=head2 preserve_params
+
+Preserve the GET and POST params in the session.
+
+=cut
+
+sub preserve_params {
+    my ($self) = @_;
+    my $conf = $self->{conf};
+
+    my $previous = $self->pnotes->{a2c}{session}{a2c}{openid}{previous} ||= { };
+    $previous->{get_args}  = $self->args;
+
+    # if we're not POSTing, just return - ??
+    return if !$self->method eq 'POST';
+
+    # get the POST body table,
+    # and get the params directly from it, so we don't mix in
+    # any GET params and keep them straight.
+    my $post_body = $self->body;
+    my @post_keys = keys %{$post_body};
+    my %post_params = map {
+        my @vals = $self->body($_);
+        ($_ => @vals > 1 ? \@vals : $vals[0]);
+    } keys %{$post_body};
+            
+    $previous->{post_params} = \%post_params;
+    return;
+}
+
 =head2 is_logged_in
 
 Check the fields in the session hash to make sure they're logged in.
@@ -704,9 +848,9 @@ Just return if not logged in, or return 1 if logged in.
 
 sub is_logged_in {
     my ($self) = @_;
-    my $sess = $self->pnotes->{session};
+    my $sess = $self->pnotes->{a2c}{session};
 
-    my $openid_sess = $self->pnotes->{session}{a2c}{openid};
+    my $openid_sess = $self->pnotes->{a2c}{session}{a2c}{openid};
 
     DEBUG sub { "openid part of session is ".Dump($openid_sess) };
 
@@ -715,10 +859,13 @@ sub is_logged_in {
     my $last_accessed = $openid_sess->{last_accessed_time};
     return if !defined $last_accessed;
 
-    my $conf = $self->{a2c_openid_conf};
+    my $conf = $self->{conf};
     my $timeout = $conf->{timeout};
 
-    return 1 if $timeout eq 'no timeout';
+    if ($timeout eq 'no timeout') {
+        $self->user($self->{uname});
+        return 1;
+    }
 
     my $current_time = time;
 
@@ -726,10 +873,11 @@ sub is_logged_in {
 
     if ($current_time - $timeout > $last_accessed) {
         DEBUG "login session has timed out";
-        $self->pnotes->{session}{a2c}{openid}{previous_uri} = $self->uri;
+        $self->pnotes->{a2c}{session}{a2c}{openid}{previous_uri} = $self->uri;
         return;
     }
 
+    $self->user($self->{uname});
     return 1;
 }
 
@@ -742,8 +890,7 @@ Log the user out by clearing the relevant fields in the session hash.
 sub logout {
     my ($self) = @_;
 
-    my $sess = $self->pnotes->{session};
-    delete @{ $self->pnotes->{session}{a2c}{openid} }{qw(
+    delete @{ $self->pnotes->{a2c}{session}{a2c}{openid} }{qw(
         logged_in
         last_accessed_time
         openid_url
